#!/usr/bin/env python
# setup.py
# setup program for clips module

# (c) 2002-2005 Francesco Garosi/JKS
# The author's copyright is expressed through the following notice, thus
# giving effective rights to copy and use this software to anyone, as shown
# in the license text.
#
# NOTICE:
#  This software is released under the terms of the GNU Lesser General Public
#  license; a copy of the text has been released with this package (see file
#  _license.py, where the license text also appears), and can be found on the
#  GNU web site, at the following address:
#
#           http://www.gnu.org/copyleft/lesser.html
#
#  Please refer to the license text for any license information. This notice
#  has to be considered part of the license, and should be kept on every copy
#  integral or modified, of the source files. The removal of the reference to
#  the license will be considered an infringement of the license itself.


"""PyCLIPS
A Python module to interface the CLIPS expert system shell library."""


__revision__ = "$Id: setup.py 248 2005-02-23 18:23:03Z Franz $"


from distutils.core import setup, Extension
from glob import glob
import os, sys, re, tempfile


# a shortcut, since it appears many times
_p = os.path.join

# find Clips source directory and zip file
zl, zd = os.listdir("."), {}
for x in zl: zd[x.lower()] = x
try: ClipsSrcZIP = _p('.', zd['clipssrc.zip'])
except: ClipsSrcZIP = "clipssrc.zip"    # will obviously produce an error
ClipsLIB_dir = _p('.', 'clipssrc')


# make a new setup.h based on current platform (note: this file is a copy
#  of the original setup.h file, and is subject to the license terms that
#  can be found in all the original CLIPS source files). This is the only
#  chunk of code which may not be subject to the same license as the others
#  (although all of this code is release under an Open Source license)
setup_h_templ = """\
/*************************************************************/
/* Principal Programmer(s):                                  */
/*      Gary D. Riley                                        */
/*      Brian L. Donnell                                     */
/* Contributing Programmer(s):                               */
/* Revision History:                                         */
/*************************************************************/
#ifndef _H_setup
#define _H_setup

#define GENERIC 0
#define UNIX_V  %s
#define UNIX_7  0
#define MAC_MCW 0
#define IBM_MCW 0
#define IBM_MSC %s
#define IBM_TBC 0
#define IBM_GCC %s

#define IBM_ZTC 0
#define IBM_ICB 0
#define IBM_SC  0
#define MAC_SC6 0
#define MAC_SC7 0
#define MAC_SC8 0
#define MAC_MPW 0
#define VAX_VMS 0
#define MAC_XCD 0

#if IBM_ZTC || IBM_MSC || IBM_TBC || IBM_ICB || IBM_SC || IBM_MCW
#define IBM 1
#else
#define IBM 0
#endif

#if MAC_SC6 || MAC_SC7 || MAC_SC8
#define MAC_SC 1
#else
#define MAC_SC 0
#endif

#if MAC_SC || MAC_MPW || MAC_MCW || MAC_XCD
#define MAC 1
#else
#define MAC 0
#endif


#if CLIPS_MAJOR >= 6

#if CLIPS_MINOR >= 22

#define VOID     void
#define VOID_ARG void
#define STD_SIZE size_t

#define BOOLEAN int
#define globle

#define RUN_TIME 0
#define DEFRULE_CONSTRUCT 1
#define CONFLICT_RESOLUTION_STRATEGIES 1
#define DYNAMIC_SALIENCE 1
#define INCREMENTAL_RESET 1
#define LOGICAL_DEPENDENCIES  1
#define DEFMODULE_CONSTRUCT 1
#define DEFTEMPLATE_CONSTRUCT 1
#define DEFFACTS_CONSTRUCT 1
#define DEFGLOBAL_CONSTRUCT 1
#define DEFFUNCTION_CONSTRUCT 1
#define DEFGENERIC_CONSTRUCT 1
#define IMPERATIVE_METHODS 1
#define OBJECT_SYSTEM 1
#define DEFINSTANCES_CONSTRUCT 1
#define IMPERATIVE_MESSAGE_HANDLERS 1
#define AUXILIARY_MESSAGE_HANDLERS 1
#define INSTANCE_SET_QUERIES 1
#define INSTANCE_PATTERN_MATCHING 1
#define BLOAD_INSTANCES 1
#define BSAVE_INSTANCES 1
#define EX_MATH 1
#define TEXTPRO_FUNCTIONS 1
#define HELP_FUNCTIONS 0
#define BLOAD_ONLY 0
#define BLOAD 0
#define BLOAD_AND_BSAVE 1
#define EMACS_EDITOR 0
#define CONSTRUCT_COMPILER 0
#define BASIC_IO 1
#define EXT_IO 1
#define STRING_FUNCTIONS 1
#define MULTIFIELD_FUNCTIONS 1
#define DEBUGGING_FUNCTIONS 1
#define PROFILING_FUNCTIONS 1
#define BLOCK_MEMORY 0
#define WINDOW_INTERFACE 1
#define SHORT_LINK_NAMES 0
#define ENVIRONMENT_API_ONLY 0
#define ALLOW_ENVIRONMENT_GLOBALS 1
#define DEVELOPER 0

#if CLIPS_MINOR >= 23
#define FACT_SET_QUERIES 1
#endif  /* CLIPS_MINOR >= 23 */


#else   /* CLIPS_MINOR >= 22 */
#error "Cannot build using CLIPS version less than 6.22"
#endif

#else   /* CLIPS_MAJOR >= 6 */
#error "Cannot build using CLIPS version less than 6.22"
#endif


#include "envrnmnt.h"

#define Bogus(x)
#define PrintCLIPS(x,y) EnvPrintRouter(GetCurrentEnvironment(),x,y)
#define GetcCLIPS(x,y) EnvGetcRouter(GetCurrentEnvironment(),x)
#define UngetcCLIPS(x,y) EnvUngetcRouter(GetCurrentEnvironment(),x,y)
#define ExitCLIPS(x) EnvExitRouter(GetCurrentEnvironment(),x)
#define CLIPSSystemError(x,y) SystemError(x,y)
#define CLIPSFunctionCall(x,y,z) FunctionCall(x,y,z)
#define InitializeCLIPS() InitializeEnvironment()
#define WCLIPS WPROMPT
#define CLIPSTrueSymbol SymbolData(GetCurrentEnvironment())->TrueSymbol
#define CLIPSFalseSymbol SymbolData(GetCurrentEnvironment())->FalseSymbol
#define EnvCLIPSTrueSymbol(theEnv) SymbolData(theEnv)->TrueSymbol
#define EnvCLIPSFalseSymbol(theEnv) SymbolData(theEnv)->FalseSymbol
#define CLIPS_FALSE 0
#define CLIPS_TRUE 1

#include "usrsetup.h"

#endif	/* _H_setup */
"""

##clips_h = """
###ifndef __PYTHON_H_CLIPS__
###define __PYTHON_H_CLIPS__
###include <Python.h>         /* in case there are some redefinitions */
###endif /* __PYTHON_H_CLIPS__ */
##"""


# The following part includes code to convert the non-environment submodule
#  to the environment-aware submodule, which defines the Environment class.

import_symbols = [
    'BREADTH_STRATEGY', 'CLIPS_MAJOR', 'CLIPS_MINOR',
    'COMPLEXITY_STRATEGY', 'CONSERVATION_MODE', 'CONVENIENCE_MODE',
    'ClipsError', 'DEPTH_STRATEGY', 'EVERY_CYCLE', 'EXTERNAL_ADDRESS',
    'FACT_ADDRESS', 'FLOAT', 'INSTANCE_ADDRESS', 'INSTANCE_NAME',
    'INTEGER', 'LEX_STRATEGY', 'LOCAL_SAVE', 'MEA_STRATEGY',
    'MULTIFIELD', 'RANDOM_STRATEGY', 'SIMPLICITY_STRATEGY', 'STRING',
    'SYMBOL', 'VISIBLE_SAVE', 'WHEN_ACTIVATED', 'WHEN_DEFINED',
    '__doc__', '__file__', '__name__', '__revision__',
    'addClearFunction', 'addEnvironmentCleanupFunction',
    'addPeriodicFunction', 'addPythonFunction', 'addResetFunction',
    'addRunFunction', 'agenda', 'allocateEnvironmentData', 'assertFact',
    'assertString', 'assignFactSlotDefaults', 'batchStar',
    'binaryLoadInstances', 'binarySaveInstances', 'bload',
    'browseClasses', 'bsave', 'build', 'classAbstractP',
    'classReactiveP', 'classSlots', 'classSubclasses',
    'classSuperclasses', 'clear', 'clearFocusStack',
    'clearPythonFunctions', 'createEnvironment', 'createFact',
    'createRawInstance', 'decrementFactCount', 'decrementInstanceCount',
    'defclassModule', 'deffactsModule', 'deffunctionModule',
    'defgenericModule', 'defglobalModule', 'definstancesModule',
    'defruleHasBreakpoint', 'defruleModule', 'deftemplateModule',
    'deleteActivation', 'deleteInstance', 'describeClass',
    'destroyEnvironment', 'directGetSlot', 'directPutSlot',
    'dribbleActive', 'dribbleOff', 'dribbleOn', 'env_addClearFunction',
    'env_addPeriodicFunction', 'env_addResetFunction',
    'env_addRunFunction', 'env_agenda', 'env_assertFact',
    'env_assertString', 'env_assignFactSlotDefaults', 'env_batchStar',
    'env_binaryLoadInstances', 'env_binarySaveInstances', 'env_bload',
    'env_browseClasses', 'env_bsave', 'env_build', 'env_classAbstractP',
    'env_classReactiveP', 'env_classSlots', 'env_classSubclasses',
    'env_classSuperclasses', 'env_clear', 'env_clearFocusStack',
    'env_createFact', 'env_createRawInstance', 'env_decrementFactCount',
    'env_decrementInstanceCount', 'env_defclassModule',
    'env_deffactsModule', 'env_deffunctionModule',
    'env_defgenericModule', 'env_defglobalModule',
    'env_definstancesModule', 'env_defruleHasBreakpoint',
    'env_defruleModule', 'env_deftemplateModule', 'env_deleteActivation',
    'env_deleteInstance', 'env_describeClass', 'env_directGetSlot',
    'env_directPutSlot', 'env_dribbleActive', 'env_dribbleOff',
    'env_dribbleOn', 'env_eval', 'env_factExistp', 'env_factIndex',
    'env_factSlotNames', 'env_facts', 'env_findDefclass',
    'env_findDeffacts', 'env_findDeffunction', 'env_findDefgeneric',
    'env_findDefglobal', 'env_findDefinstances',
    'env_findDefmessageHandler', 'env_findDefmodule', 'env_findDefrule',
    'env_findDeftemplate', 'env_findInstance', 'env_focus',
    'env_functionCall', 'env_getActivationName',
    'env_getActivationPPForm', 'env_getActivationSalience',
    'env_getAgendaChanged', 'env_getAutoFloatDividend',
    'env_getClassDefaultsMode', 'env_getCurrentModule',
    'env_getDefclassList', 'env_getDefclassName',
    'env_getDefclassPPForm', 'env_getDefclassWatchInstances',
    'env_getDefclassWatchSlots', 'env_getDeffactsList',
    'env_getDeffactsName', 'env_getDeffactsPPForm',
    'env_getDeffunctionList', 'env_getDeffunctionName',
    'env_getDeffunctionPPForm', 'env_getDeffunctionWatch',
    'env_getDefgenericList', 'env_getDefgenericName',
    'env_getDefgenericPPForm', 'env_getDefgenericWatch',
    'env_getDefglobalList', 'env_getDefglobalName',
    'env_getDefglobalPPForm', 'env_getDefglobalValue',
    'env_getDefglobalValueForm', 'env_getDefglobalWatch',
    'env_getDefinstancesList', 'env_getDefinstancesName',
    'env_getDefinstancesPPForm', 'env_getDefmessageHandlerList',
    'env_getDefmessageHandlerName', 'env_getDefmessageHandlerPPForm',
    'env_getDefmessageHandlerType', 'env_getDefmessageHandlerWatch',
    'env_getDefmethodDescription', 'env_getDefmethodList',
    'env_getDefmethodPPForm', 'env_getDefmethodWatch',
    'env_getDefmoduleList', 'env_getDefmoduleName',
    'env_getDefmodulePPForm', 'env_getDefruleList', 'env_getDefruleName',
    'env_getDefrulePPForm', 'env_getDefruleWatchActivations',
    'env_getDefruleWatchFirings', 'env_getDeftemplateList',
    'env_getDeftemplateName', 'env_getDeftemplatePPForm',
    'env_getDeftemplateWatch', 'env_getDynamicConstraintChecking',
    'env_getFactDuplication', 'env_getFactList',
    'env_getFactListChanged', 'env_getFactPPForm', 'env_getFactSlot',
    'env_getFocus', 'env_getFocusStack', 'env_getGlobalsChanged',
    'env_getIncrementalReset', 'env_getInstanceClass',
    'env_getInstanceName', 'env_getInstancePPForm',
    'env_getInstancesChanged', 'env_getMethodRestrictions',
    'env_getNextActivation', 'env_getNextDefclass',
    'env_getNextDeffacts', 'env_getNextDeffunction',
    'env_getNextDefgeneric', 'env_getNextDefglobal',
    'env_getNextDefinstances', 'env_getNextDefmessageHandler',
    'env_getNextDefmethod', 'env_getNextDefmodule', 'env_getNextDefrule',
    'env_getNextDeftemplate', 'env_getNextFact',
    'env_getNextFactInTemplate', 'env_getNextInstance',
    'env_getNextInstanceInClass',
    'env_getNextInstanceInClassAndSubclasses', 'env_getResetGlobals',
    'env_getSalienceEvaluation', 'env_getSequenceOperatorRecognition',
    'env_getStaticConstraintChecking', 'env_getStrategy',
    'env_getWatchItem', 'env_incrementFactCount',
    'env_incrementInstanceCount', 'env_instances',
    'env_isDefclassDeletable', 'env_isDeffactsDeletable',
    'env_isDeffunctionDeletable', 'env_isDefgenericDeletable',
    'env_isDefglobalDeletable', 'env_isDefinstancesDeletable',
    'env_isDefmessageHandlerDeletable', 'env_isDefmethodDeletable',
    'env_isDefruleDeletable', 'env_isDeftemplateDeletable',
    'env_listDefclasses', 'env_listDeffacts', 'env_listDeffunctions',
    'env_listDefgenerics', 'env_listDefglobals', 'env_listDefinstances',
    'env_listDefmessageHandlers', 'env_listDefmethods',
    'env_listDefmodules', 'env_listDefrules', 'env_listDeftemplates',
    'env_listFocusStack', 'env_load', 'env_loadFacts',
    'env_loadFactsFromString', 'env_loadInstances',
    'env_loadInstancesFromString', 'env_makeInstance', 'env_matches',
    'env_popFocus', 'env_previewSend', 'env_putFactSlot', 'env_refresh',
    'env_refreshAgenda', 'env_removeBreak', 'env_removeClearFunction',
    'env_removePeriodicFunction', 'env_removeResetFunction',
    'env_removeRunFunction', 'env_reorderAgenda', 'env_reset',
    'env_restoreInstances', 'env_restoreInstancesFromString',
    'env_retract', 'env_run', 'env_save', 'env_saveFacts',
    'env_saveInstances', 'env_send', 'env_sendCommand',
    'env_setActivationSalience', 'env_setAgendaChanged',
    'env_setAutoFloatDividend', 'env_setBreak',
    'env_setClassDefaultsMode', 'env_setCurrentModule',
    'env_setDefclassWatchInstances', 'env_setDefclassWatchSlots',
    'env_setDeffunctionWatch', 'env_setDefgenericWatch',
    'env_setDefglobalValue', 'env_setDefglobalWatch',
    'env_setDefmessageHandlerWatch', 'env_setDefmethodWatch',
    'env_setDefruleWatchActivations', 'env_setDefruleWatchFirings',
    'env_setDeftemplateWatch', 'env_setDynamicConstraintChecking',
    'env_setFactDuplication', 'env_setFactListChanged',
    'env_setGlobalsChanged', 'env_setIncrementalReset',
    'env_setInstancesChanged', 'env_setResetGlobals',
    'env_setSalienceEvaluation', 'env_setSequenceOperatorRecognition',
    'env_setStaticConstraintChecking', 'env_setStrategy',
    'env_showBreaks', 'env_showDefglobals', 'env_slotAllowedValues',
    'env_slotCardinality', 'env_slotDirectAccessP', 'env_slotExistP',
    'env_slotFacets', 'env_slotInitableP', 'env_slotPublicP',
    'env_slotRange', 'env_slotSources', 'env_slotTypes',
    'env_slotWritableP', 'env_subclassP', 'env_superclassP',
    'env_undefclass', 'env_undeffacts', 'env_undeffunction',
    'env_undefgeneric', 'env_undefglobal', 'env_undefinstances',
    'env_undefmessageHandler', 'env_undefmethod', 'env_undefrule',
    'env_undeftemplate', 'env_unmakeInstance', 'env_unwatch',
    'env_validInstanceAddress', 'env_watch', 'eval', 'factExistp',
    'factIndex', 'factSlotNames', 'facts', 'findDefclass',
    'findDeffacts', 'findDeffunction', 'findDefgeneric', 'findDefglobal',
    'findDefinstances', 'findDefmessageHandler', 'findDefmodule',
    'findDefrule', 'findDeftemplate', 'findInstance', 'focus',
    'functionCall', 'getActivationName', 'getActivationPPForm',
    'getActivationSalience', 'getAgendaChanged', 'getAutoFloatDividend',
    'getClassDefaultsMode', 'getConserveMemory', 'getCurrentEnvironment',
    'getCurrentModule', 'getDefclassList', 'getDefclassName',
    'getDefclassPPForm', 'getDefclassWatchInstances',
    'getDefclassWatchSlots', 'getDeffactsList', 'getDeffactsName',
    'getDeffactsPPForm', 'getDeffunctionList', 'getDeffunctionName',
    'getDeffunctionPPForm', 'getDeffunctionWatch', 'getDefgenericList',
    'getDefgenericName', 'getDefgenericPPForm', 'getDefgenericWatch',
    'getDefglobalList', 'getDefglobalName', 'getDefglobalPPForm',
    'getDefglobalValue', 'getDefglobalValueForm', 'getDefglobalWatch',
    'getDefinstancesList', 'getDefinstancesName',
    'getDefinstancesPPForm', 'getDefmessageHandlerList',
    'getDefmessageHandlerName', 'getDefmessageHandlerPPForm',
    'getDefmessageHandlerType', 'getDefmessageHandlerWatch',
    'getDefmethodDescription', 'getDefmethodList', 'getDefmethodPPForm',
    'getDefmethodWatch', 'getDefmoduleList', 'getDefmoduleName',
    'getDefmodulePPForm', 'getDefruleList', 'getDefruleName',
    'getDefrulePPForm', 'getDefruleWatchActivations',
    'getDefruleWatchFirings', 'getDeftemplateList', 'getDeftemplateName',
    'getDeftemplatePPForm', 'getDeftemplateWatch',
    'getDynamicConstraintChecking', 'getEnvironmentData',
    'getEnvironmentIndex', 'getFactDuplication', 'getFactList',
    'getFactListChanged', 'getFactPPForm', 'getFactSlot', 'getFocus',
    'getFocusStack', 'getGlobalsChanged', 'getIncrementalReset',
    'getInstanceClass', 'getInstanceName', 'getInstancePPForm',
    'getInstancesChanged', 'getMethodRestrictions', 'getNextActivation',
    'getNextDefclass', 'getNextDeffacts', 'getNextDeffunction',
    'getNextDefgeneric', 'getNextDefglobal', 'getNextDefinstances',
    'getNextDefmessageHandler', 'getNextDefmethod', 'getNextDefmodule',
    'getNextDefrule', 'getNextDeftemplate', 'getNextFact',
    'getNextFactInTemplate', 'getNextInstance', 'getNextInstanceInClass',
    'getNextInstanceInClassAndSubclasses', 'getPPBufferSize',
    'getPrintExternalTraceback', 'getResetGlobals',
    'getSalienceEvaluation', 'getSequenceOperatorRecognition',
    'getStaticConstraintChecking', 'getStrategy', 'getWatchItem',
    'incrementFactCount', 'incrementInstanceCount', 'instances',
    'isActivation', 'isDefclass', 'isDefclassDeletable', 'isDeffacts',
    'isDeffactsDeletable', 'isDeffunction', 'isDeffunctionDeletable',
    'isDefgeneric', 'isDefgenericDeletable', 'isDefglobal',
    'isDefglobalDeletable', 'isDefinstances', 'isDefinstancesDeletable',
    'isDefmessageHandlerDeletable', 'isDefmethod',
    'isDefmethodDeletable', 'isDefmodule', 'isDefrule',
    'isDefruleDeletable', 'isDeftemplate', 'isDeftemplateDeletable',
    'isEnvironment', 'isFact', 'isInstance', 'listDefclasses',
    'listDeffacts', 'listDeffunctions', 'listDefgenerics',
    'listDefglobals', 'listDefinstances', 'listDefmessageHandlers',
    'listDefmethods', 'listDefmodules', 'listDefrules',
    'listDeftemplates', 'listFocusStack', 'load', 'loadFacts',
    'loadFactsFromString', 'loadInstances', 'loadInstancesFromString',
    'makeInstance', 'matches', 'memRequests', 'memUsed', 'popFocus',
    'previewSend', 'putFactSlot', 'refresh', 'refreshAgenda',
    'releaseMem', 'removeBreak', 'removeClearFunction',
    'removePeriodicFunction', 'removePythonFunction',
    'removeResetFunction', 'removeRunFunction', 'reorderAgenda', 'reset',
    'restoreInstances', 'restoreInstancesFromString', 'retract',
    'routerClear', 'routerRead', 'routerReadline', 'routerWrite', 'run',
    'save', 'saveFacts', 'saveInstances', 'send', 'sendCommand',
    'setActivationSalience', 'setAgendaChanged', 'setAutoFloatDividend',
    'setBreak', 'setClassDefaultsMode', 'setConserveMemory',
    'setCurrentEnvironment', 'setCurrentEnvironmentByIndex',
    'setCurrentModule', 'setDefclassWatchInstances',
    'setDefclassWatchSlots', 'setDeffunctionWatch', 'setDefgenericWatch',
    'setDefglobalValue', 'setDefglobalWatch',
    'setDefmessageHandlerWatch', 'setDefmethodWatch',
    'setDefruleWatchActivations', 'setDefruleWatchFirings',
    'setDeftemplateWatch', 'setDynamicConstraintChecking',
    'setFactDuplication', 'setFactListChanged', 'setGlobalsChanged',
    'setIncrementalReset', 'setInstancesChanged',
    'setOutOfMemoryFunction', 'setPPBufferSize',
    'setPrintExternalTraceback', 'setResetGlobals',
    'setSalienceEvaluation', 'setSequenceOperatorRecognition',
    'setStaticConstraintChecking', 'setStrategy', 'showBreaks',
    'showDefglobals', 'slotAllowedValues', 'slotCardinality',
    'slotDirectAccessP', 'slotExistP', 'slotFacets', 'slotInitableP',
    'slotPublicP', 'slotRange', 'slotSources', 'slotTypes',
    'slotWritableP', 'subclassP', 'superclassP', 'undefclass',
    'undeffacts', 'undeffunction', 'undefgeneric', 'undefglobal',
    'undefinstances', 'undefmessageHandler', 'undefmethod', 'undefrule',
    'undeftemplate', 'unmakeInstance', 'unwatch', 'validInstanceAddress',
    'watch',
    ]



HEADER = '''\
# _eclips_wrap.py
# environment aware functions for CLIPS, embedded in an Environment class

# (c) 2002-2003 Francesco Garosi/JKS
# The Author's copyright is expressed through the following notice, thus
# giving actual rights to copy and use this software to anyone, as expressed
# in the license text.
#
# NOTICE:
# This software is released under the terms of the GNU Lesser General Public
#  license; a copy of the text has been released with this package (see file
#  license.txt), and can be found on the GNU web site, at the following
#  address:
#
#           http://www.gnu.org/copyleft/lesser.html
#
#  Please refer to the license text for any license information. This notice
#  has to be considered part of the license, and should be kept on every copy
#  integral or modified, of the source files. The removal of the reference to
#  the license will be considered an infringement of the license itself.

"""\
clips - high-level interface to the CLIPS engine module
        (c) 2002-2005 Francesco Garosi/JKS
"""

# standard imports
import sys as _sys

import os as  _os
import types as _types

# the low-level module
import _clips as _c


# ========================================================================== #
# globals

# bring the CLIPS Exception object at top level
ClipsError = _c.ClipsError


# redeclare manifest constants here in order to avoid having to
#  reference the ones defined in te low-level module _clips

# These manifest constants are commented out, since the user has to rely
#  on the class constructor defined below in order to build values to
#  pass to the CLIPS engine
# check Python version, and issue a n exception
if _sys.version[:3] < "2.2":
    raise _c.ClipsError, "M99: Python 2.2 or higher required"


# these globals are redefined instead of reimported for sake of speed
LOCAL_SAVE = _c.LOCAL_SAVE
VISIBLE_SAVE = _c.VISIBLE_SAVE

WHEN_DEFINED = _c.WHEN_DEFINED
WHEN_ACTIVATED = _c.WHEN_ACTIVATED
EVERY_CYCLE = _c.EVERY_CYCLE

DEPTH_STRATEGY = _c.DEPTH_STRATEGY
BREADTH_STRATEGY = _c.BREADTH_STRATEGY
LEX_STRATEGY = _c.LEX_STRATEGY
MEA_STRATEGY = _c.MEA_STRATEGY
COMPLEXITY_STRATEGY = _c.COMPLEXITY_STRATEGY
SIMPLICITY_STRATEGY = _c.SIMPLICITY_STRATEGY
RANDOM_STRATEGY = _c.RANDOM_STRATEGY

CONVENIENCE_MODE = _c.CONVENIENCE_MODE
CONSERVATION_MODE = _c.CONSERVATION_MODE


# import adequate symbols from _clips_wrap
from _clips_wrap import Nil, Integer, Float, String, Symbol, InstanceName, \\
                        Multifield, _cl2py, _py2cl, _py2clsyntax, \\
                        ClipsIntegerType, ClipsFloatType, ClipsStringType, \\
                        ClipsSymbolType, ClipsInstanceNameType, \\
                        ClipsMultifieldType, ClipsNilType, \\
                        _setStockClasses, AROUND, BEFORE, PRIMARY, AFTER



# environment class:
class Environment(object):
    """class representing an environment: implements all global classes"""

'''


INIT = '''

    # constructor possibly sets the "borrowed" flag, to state that this
    #  is a Python class around an existing object: in this case the
    #  underlying CLIPS environment is not attempted to be destroyed on
    #  deletion
    def __init__(self, o=None):
        """environment constructor"""
        if o is None:
            self.__env = _c.createEnvironment()
            self.__borrowed = False
        else:
            if _c.isEnvironment(o):
                self.__env = o
                self.__borrowed = True
            else: raise TypeError, "invalid argument for constructor"

'''


LAST_INIT = '''\
        # if o is not None, then this is an internal object and its status
        #  should not be modified by the user, nor the stock objects be
        #  accessible for direct inspection or subclassing (as this could
        #  be the current environment and might be corrupted)
        if o is None:
            self.EngineConfig = self._clips_Status()
            self.DebugConfig = self._clips_Debug()

'''


FOOTER = '''

    def __del__(self):
        """environment destructor"""
        if not self.__borrowed:
            try: _c.destroyEnvironment(self.__env)
            except ClipsError: pass

    def __repr__(self):
        """representation of environment, borrowed by underlying object"""
        return "<Environment: " + repr(self.__env)[1:-1] + ">"

    def __property_getIndex(self):
        return _c.getEnvironmentIndex(self.__env)
    Index = property(__property_getIndex, None, None,
                     "Return index of this Environment")

    def SetCurrent(self):
        """Make this Environment the current Environment"""
        _c.setCurrentEnvironment(self.__env)
        _setStockClasses()



# A function that returns current Environment
def CurrentEnvironment():
    """Return current Environment"""
    cenv = _c.getCurrentEnvironment()
    env = Environment(cenv)
    env.EngineConfig = env._clips_Status()
    env.DebugConfig = env._clips_Debug()
    return env



# end.
'''


# this is what all the functions in the low-level module look like
func_re = re.compile(r"(_c\.\w+\()")
func_re_NA = re.compile(r"(_c\.\w+\(\))")
class_re = re.compile(r"^class\ ")
def_re = re.compile(r"def\ (\w+\()")
def_re_NA = re.compile(r"def\ (\w+\(\))")


# remove leading underscores
def rmleadus(s):
    t = s
    while t[0] == "_":
        t = t[1:]
    return t

# convert a line of code
def cvtline(s, cvtdef=False, classes=[]):
    if s.strip() == "": return ""
    if s.strip()[0] == "#": return ""
    s1 = s
    mo = func_re.search(s1)
    if mo:
        li = mo.groups()
        for x in li:
            t = x[3:-1]
            if "env_%s" % t in import_symbols:
                if func_re_NA.search(s1):
                    s1 = s1.replace(x, "_c.env_%s(self.__env" % t)
                else:
                    s1 = s1.replace(x, "_c.env_%s(self.__env, " % t)
    if cvtdef:
        mo = def_re.search(s1)
        if mo:
            if def_re_NA.search(s1):
                s1 = def_re.sub(mo.group(0) + "self", s1)
            else:
                s1 = def_re.sub(mo.group(0) + "self, ", s1)
        for x in classes:
            s1 = s1.replace(" %s(" % x, " self.%s(" % x)
            s1 = s1.replace("(%s(" % x, "(self.%s(" % x)
    return (" " * 4) + s1



# read the whole file and return it as a list of lines
def readfile(f):
    li = []
    read = False
    for x in f:
        if x.strip() == "# {{END_ENV}}": read = False
        if read:
            li.append(x.rstrip())
        if x.strip() == "# {{BEGIN_ENV}}": read = True
    return li

def readclasses(f):
    li = []
    read = False
    for x in f:
        if x.strip() == "# {{END_ENVCLASSES}}": read = False
        if read:
            li.append(x.rstrip())
        if x.strip() == "# {{BEGIN_ENVCLASSES}}": read = True
    return li


# read up a class definition and make a nested class out of it
def readClass(lines):
    def getClassBody(ll):
        body, store = [], False
        classname = ""
        for x in ll:
            if x.strip():
                if x.strip() == "# {{LAST_CLASS}}" or x.split()[0] == "class":
                    if store: return ll, classname, body
                    else:
                        classname = x.split()[1].split("(")[0]
                        store = True
                if store:
                    body.append(x)
                    ll = ll[1:]
        return [], "", []
    lines, classname, clli= getClassBody(lines)
    t = []
    for x in clli:
        if x.strip() != "" and x.strip()[0] != "#":
            t.append(x)
    return lines, classname, t

def cvtline_class(s, clname, classes=[]):
    t = cvtline(s)
    if not clname.startswith("__"):
        t = t.replace("def __init__(self", "def __fakeinit(self, env")
    return t

def cvtclass(lines, clname):
    t, num_initargs = [], 0
    for x in lines:
        t.append(cvtline_class(x, clname))
        if x.strip().startswith("def __init__") \
           and not clname.startswith("__"):
            t.append("            self.__env = env")
            num_initargs = len(x.split(","))
    return num_initargs, t

# hack to convert direct instantiations of classes in return values
def cvt_class_dirinst(s, classes=[]):
    if s.split()[0] == 'class': return s
    for x in classes:
        s = s.replace(" %s(" % x, " Environment(self.__env).%s(" % x)
        s = s.replace("(%s(" % x, "(Environment(self.__env).%s(" % x)
    return s

# convert all classes
def cvt_all_classes(li):
    extrainit, extraheader, done = [], [], []
    lambda_templ = "        self.%(name)s.__init__ = lambda s%(args)s:" \
        + " self.%(name)s._%(nname)s__fakeinit(s, self.__env%(args)s)"
    # 'done' list is a workaround, I know

    li, clname, clli = readClass(li)
    if not clname in done:
        done.append(clname)
        nargs, nli = cvtclass(clli, clname)
        largs = []
        for i in range(1, nargs):
            largs.append("a%s" % str(i).strip())
        sargs = ", ".join(largs)
        if sargs.strip() != "": sargs = ", " + sargs
        if clname[:2] != "__":
            extrainit.append(lambda_templ % {
                'nname': rmleadus(clname), 'name': clname, 'args': sargs})
        extraheader += nli
    while clname != "":
        li, clname, clli = readClass(li)
        if not clname in done:
            done.append(clname)
            if clname.strip() == "": break
            nargs, nli = cvtclass(clli, clname)
            largs = []
            for i in range(1, nargs):
                largs.append("a%s" % str(i).strip())
            sargs = ", ".join(largs)
            if sargs.strip() != "": sargs = ", " + sargs
            if clname[:2] != "__":
                extrainit.append(lambda_templ % {
                    'nname': rmleadus(clname), 'name': clname, 'args': sargs})
            extraheader += nli
    while "" in done: done.remove("")
    # ????
    done_c = filter(lambda x: x[0] != '_', done)
    extraheader1 = []
    for ei in extraheader:
        extraheader1.append(cvt_class_dirinst(ei, done_c))
    # ????
    return extrainit, extraheader1, done



# main conversion program
def convert_all():
    s = _p('clips', '_clips_wrap.py')
    f = open(s)
    li = readfile(f)
    f.close()
    f = open(s)
    clli = readclasses(f)
    f.close()
    xinit, xheader, classes = cvt_all_classes(clli)
    newfn = _p('clips', '_eclips_wrap.py')
    f = open(newfn, 'w')
    f.write(HEADER)
    for x in xheader:
        f.write(x + "\n")
    f.write(INIT)
    for x in xinit:
        f.write(x + "\n")
    f.write(LAST_INIT)
    f.write("\n\n")
    for x in li:
        s = cvtline(x, True, classes)
        if s:
            f.write(s + "\n")
    f.write(FOOTER)
    f.close()


# This retrieves the CLIPS version looking up headers
def get_clips_version(fn):
    li = open(fn).readlines()
    cre = re.compile(r"\#define\ VERSION_STRING\ \"([0-9]+\.[0-9]+)\"")
    vno = "unknown"
    for s in li:
        f = cre.search(s)
        if f:
            vno = f.groups(1)
            break
    return "%s" % vno



# Actual distutils setup routine
print "Module 'clips': Python to CLIPS interface"

# check for CLIPS library files, and eventually uncompress them from
#  the CLIPS source - to be found at CLIPS website. In case no ZIP
#  file is even present here, notice the user that it has to be
#  downloaded and put here, in the directory where this file resides
nozip_notice = """\
ERROR: setup.py could not find the CLIPS source files. These files
       are necessary to build the CLIPS interface module: please
       download the archive (%s) and copy it to the directory
       where setup.py resides. The setup program will take care of
       unpacking the necessary source files in an appropriate place.
"""

badzip_notice = """\
ERROR: setup.py could not read one or more file(s) from the source
       archive. Please provide a good copy of the archive (%s).
"""


# this is used to normalize end-of-line characters across systems, since we
#  use the ZIP version of the archive (which is for Win32/DOS systems). The
#  use of TemporaryFile is a hack, but so we can read lines letting Python
#  do the dirty job of removing bad EOLs.
def normalize_eols(t):
    def _remove_badchars(x):
        t = ""
        x = x.replace('\t', " " * 8)
        for c in x:
            if ord(c) >= 32 and ord(c) < 128:
                t += c
        return t
    tf = tempfile.TemporaryFile()
    tf.write(t)
    tf.seek(0)
    li = tf.readlines()
    tf.close()
    li = map(lambda x: x.rstrip(), li)
    li = map(_remove_badchars, li)
    return li


if not os.path.exists(ClipsLIB_dir):
    import zipfile
    try:
        print "Opening CLIPS source archive (%s)..." % ClipsSrcZIP
        zf = zipfile.ZipFile(ClipsSrcZIP)
        os.mkdir(ClipsLIB_dir)
        li = zf.namelist()
        for x in li:
            if '/' in x: n = _p(ClipsLIB_dir, x.split('/')[1])
            else: n = _p(ClipsLIB_dir, x)
            sys.stdout.write("\tExtracting %s... " % n)
            li = normalize_eols(zf.read(x))
            f = open(n, 'w')
            for t in li: f.write("%s\n" % t)
            f.close()
            sys.stdout.write("done.\n")
        zf.close()
        print "All CLIPS source files extracted, continuing build."
    except zipfile.error:
        print badzip_notice % ClipsSrcZIP
        sys.exit(2)
    except:
        print nozip_notice % ClipsSrcZIP
        sys.exit(2)



# ----------------------------------------------------------------------------
# -- 8-< -- CUT FROM HERE --------------------------------------------- >-8 --

bad_platform = """\
setup.py: setup provided only for posix/win32 systems
NOTE: If you want to provide your own setup.h file, please remove the lines
      between the CUT ... HERE markers and edit your setup.h file using the
      original file (setup.h or setup.h.ORIG). Please be cautious doing so,
      since the module expects all the constructs and functions defined in
      the ad-hoc configuration file to be available: modifying setup.h by
      yourself may lead to an inconsistent module or compilation errors.
"""
if sys.platform == "win32": defines = (0, 1, 0)
elif os.name == "posix": defines = (0, 0, 1)
else:
    sys.stderr.write(bad_platform)
    sys.exit(2)

setup_h = setup_h_templ % defines

# rename setup.h to setup.h.ORIG to keep a copy of original file; if the
#  copy already exists, we are supposed to already have built the new
#  CLIPS setup file, thus we do not
setup_orig_name = _p(ClipsLIB_dir, 'setup.h')
setup_copy_name = _p(ClipsLIB_dir, 'setup.h.ORIG')
if os.path.exists(setup_orig_name) and not os.path.exists(setup_copy_name):
    sys.stdout.write("copying setup.h to setup.h.ORIG, making new header...")
    os.rename(setup_orig_name, setup_copy_name)
    f = open(setup_orig_name, 'w')
    f.write(setup_h)
    f.close()
    sys.stdout.write("Done.\n")
### this should set up almost all changes Python makes to standard symbols
##clipsh_orig_name = _p(ClipsLIB_dir, 'clips.h')
##clipsh_copy_name = _p(ClipsLIB_dir, 'clips.h.ORIG')
##if os.path.exists(clipsh_orig_name) and not os.path.exists(clipsh_copy_name):
##    sys.stdout.write("copying clips.h to clips.h.ORIG, making new header...")
##    f = open(clipsh_orig_name)
##    s = f.read()
##    f.close()
##    os.rename(clipsh_orig_name, clipsh_copy_name)
##    f = open(clipsh_orig_name, 'w')
##    f.write(s + "\n" + clips_h)
##    f.close()
##    sys.stdout.write("Done.\n")
# here we remove the (exit) function from CLIPS to prevent aborting Python
routerc_orig_name = _p(ClipsLIB_dir, 'router.c')
routerc_copy_name = _p(ClipsLIB_dir, 'router.c.ORIG')
if os.path.exists(routerc_orig_name) and not os.path.exists(routerc_copy_name):
    sys.stdout.write("copying router.c to router.c.ORIG, making new source...")
    f = open(routerc_orig_name)
    li = f.readlines()
    f.close()
    os.rename(routerc_orig_name, routerc_copy_name)
    badline_re = re.compile(
        "\s*EnvDefineFunction2.+exit.+ExitCommand.+ExitCommand.+")
    f = open(routerc_orig_name, 'w')
    for l in li:
        if badline_re.match(l):
            f.write("/* INTENTIONALLY SKIPPED */\n")
            f.write("#ifndef PYCLIPS\n")
            f.write(l)
            f.write("#endif /* PYCLIPS */\n")
        else: f.write(l)
    f.close()
    sys.stdout.write("Done.\n")

# should hack constant.h: FLOAT #defined to be 0 can lead to misunderstandings
# unfortunately we cannot do it, since the type constants are quite stuck
# in their position...
##consth_orig_name = _p(ClipsLIB_dir, 'constant.h')
##consth_copy_name = _p(ClipsLIB_dir, 'constant.h.ORIG')
##if os.path.exists(consth_orig_name) and not os.path.exists(consth_copy_name):
##    reFLOAT = re.compile("^\#define\s+FLOAT\s+0$")
##    reINTEGER = re.compile("^\#define\s+INTEGER\s+1$")
##    reSYMBOL = re.compile("^\#define\s+SYMBOL\s+2$")
##    reSTRING = re.compile("^\#define\s+STRING\s+3$")
##    reMULTIFIELD = re.compile("^\#define\s+MULTIFIELD\s+4$")
##    reEXTERNAL_ADDRESS = re.compile("^\#define\s+EXTERNAL_ADDRESS\s+5$")
##    reFACT_ADDRESS = re.compile("^\#define\s+FACT_ADDRESS\s+6$")
##    reINSTANCE_ADDRESS = re.compile("^\#define\s+INSTANCE_ADDRESS\s+7$")
##    reINSTANCE_NAME = re.compile("^\#define\s+INSTANCE_NAME\s+8$")
##    sys.stdout.write("copying constant.h to constant.h.ORIG, making new header...")
##    f = open(consth_orig_name)
##    li = f.readlines()
##    f.close()
##    os.rename(consth_orig_name, consth_copy_name)
##    f = open(consth_orig_name, 'w')
##    for x in li:
##        if reFLOAT.match(x): f.write("#define FLOAT 201\n")
##        elif reINTEGER.match(x): f.write("#define INTEGER 202\n")
##        elif reSYMBOL.match(x): f.write("#define SYMBOL 203\n")
##        elif reSTRING.match(x): f.write("#define STRING 204\n")
##        elif reMULTIFIELD.match(x): f.write("#define MULTIFIELD 205\n")
##        elif reEXTERNAL_ADDRESS.match(x): f.write("#define EXTERNAL_ADDRESS 206\n")
##        elif reFACT_ADDRESS.match(x): f.write("#define FACT_ADDRESS 207\n")
##        elif reINSTANCE_ADDRESS.match(x): f.write("#define INSTANCE_ADDRESS 208\n")
##        elif reINSTANCE_NAME.match(x): f.write("#define INSTANCE_NAME 209\n")
##        else: f.write(x)
##    f.close()
##    sys.stdout.write("Done.\n")


# -- 8-< -- CUT UP TO HERE -------------------------------------------- >-8 --
# ----------------------------------------------------------------------------



# remove main.c since it interferes with clipsmodule.c
all_clipssrc = glob(_p(ClipsLIB_dir, '*.c'))
main_clipssrc = ['clipsmodule.c', 'matches_py.c', 'clear_py.c',
                 'inscptch_py.c']
for x in all_clipssrc:
    if os.path.basename(x) == 'main.c':
        all_clipssrc.remove(x)

# build "companion" module
print "Setup revision: %s" % __revision__
sys.stdout.write("building environment-aware submodule... ")
convert_all()
sys.stdout.write("Done!\n")

# retrieve used CLIPS version
clips_version = get_clips_version(_p("clipssrc", "constant.h"))
print "Found CLIPS version: %s" % clips_version
maj, min = clips_version.split('.', 1)
CFLAGS = [
    '-DPYCLIPS',
    '-DCLIPS_MAJOR=%s' % maj,
    '-DCLIPS_MINOR=%s' % min,
    ]


# start setup
print "Standard setup in progress:"
setup(name="pyclips",
      version="1.0-clips_%s" % clips_version,
      description="Python CLIPS interface",
      long_description=__doc__,
      author="Francesco Garosi",
      author_email="franzg@users.sourceforge.net",
      url="http://pyclips.sourceforge.net",
      packages=['clips'],
      ext_modules=[
          Extension('clips._clips',
                    main_clipssrc + all_clipssrc,
                    extra_compile_args=CFLAGS,
                    include_dirs=[ClipsLIB_dir]),
          ],
     )

# end.
